try:
    import redis
except ImportError:
    # We can allow custom provider only usage without redis-py being installed
    redis = None

__all__ = ('FlaskRedis', )
__version__ = '0.3.0'


class FlaskRedis(object):

    def __init__(self, app=None, strict=True, config_prefix='REDIS', **kwargs):
        self._redis_client = None
        self.provider_class = redis.StrictRedis if strict else redis.Redis
        self.provider_kwargs = kwargs
        self.config_prefix = config_prefix

        if app is not None:
            self.init_app(app)

    @classmethod
    def from_custom_provider(cls, provider, app=None, **kwargs):
        assert provider is not None, 'your custom provider is None, come on'

        # We never pass the app parameter here, so we can call init_app
        # ourselves later, after the provider class has been set
        instance = cls(**kwargs)

        instance.provider_class = provider
        if app is not None:
            instance.init_app(app)
        return instance

    def init_app(self, app, **kwargs):
        redis_url = app.config.get(
            '{0}_URL'.format(self.config_prefix), 'redis://localhost:6379/0'
        )

        self.provider_kwargs.update(kwargs)
        self._redis_client = self.provider_class.from_url(
            redis_url, **self.provider_kwargs
        )

        if not hasattr(app, 'extensions'):
            app.extensions = {}
        app.extensions[self.config_prefix.lower()] = self

    def __getattr__(self, name):
        return getattr(self._redis_client, name)

    def __getitem__(self, name):
        return self._redis_client.get(name)

    def __setitem__(self, name, value, expire):
        self._redis_client.set(name, value, expire)

    def __delitem__(self, name):
        self._redis_client.delete(name)

    def flushdb(self):
        self._redis_client.flushdb()



import os
import math
import json
import random
import string
import requests
from io import BytesIO
from base_aios.aios_part_uploader.exceptions import FileTransError

class CSResponse(object):
    ok = False
    text = None

    def __init__(self, ok, text):
        self.ok = ok
        self.text = text

class Uploader(object):
    r"""文件存储服务NFS上传接口  
    params:sub_dir 文件在文件系统存放的目录  
    例如: 挂载点是/mnt/, sub_dir是/dev/dmp/1/..., file_name是c:\\test.txt
        最终会存放到的地址: 
            /mnt/dev/dmp/1/.../test.txt
        格式为<挂载点>/<中间路径>/<文件名>
        中间路径格式：
            <env>/<自定义>
        env是环境标记:
            prod:生产环境
            test:测试环境
            dev:开发环境
    params:file_name 物理文件路径  
    params:args 上传接口 upload_url
    
    文件分片上传阶段异常  
    FileTransError : File transfer failed (1)! xxx  

    分片合并阶段异常  
    FileTransError : File transfer failed (2)! yyy  
    """

    def __init__(self, sub_dir, file_name, **args):
        if not sub_dir:
            raise Exception('sub_dir is missing or invalidate!')
        if not os.path.exists(file_name):
            raise Exception('File not exist!')

        self._upload_url = args.get('upload_url')

        if not self._upload_url:
            raise Exception('upload_url is required!')
        if not os.path.basename(file_name):
            raise Exception('upload_url is invalid format!')

        _jwt_token = args.get('jwt_token')
        
        self._headers = None
        if _jwt_token:
          self._headers = {'Authorization': _jwt_token}
        
        self._file_name = file_name
        self._sub_dir = sub_dir

    def start(self):
        try:
            self._reset_task_id()
            self._push_part_file()

            return CSResponse(True, self.text)
        except Exception as err:
            return CSResponse(False, str(err))

    def _reset_task_id(self):
        self.task_id = 'wu_{}'.format(
            ''.join(random.sample(string.ascii_letters + string.digits, 28)))

    # slice
    def _push_part_file(self):

        _chunk = 1
        _file_name = os.path.basename(self._file_name)
        _upload_data = {'task_id': self.task_id, 'sub_dir': self._sub_dir, 'filename': _file_name}

        _chunk_size = 20 * 1024 * 1024
        _totalChunks = math.ceil(os.path.getsize(self._file_name) / _chunk_size)
        
        with open(self._file_name, 'rb') as f:
            while True:
                # 每次读取20M
                part_bytes = f.read(_chunk_size)
                if not part_bytes:
                    break

                _upload_data['chunkNumber'] = _chunk
                _upload_data['totalChunks'] = _totalChunks

                _file_name = os.path.basename(self._file_name)

                _files = {'file': (_file_name, BytesIO(part_bytes), 'application')}
                
                _res = requests.post(self._upload_url, files=_files, data=_upload_data, headers=self._headers)
                
                if not _res.ok and _res.status_code == 200:
                    raise FileTransError('File transfer failed (1)! {}'.format(_res.text))
                _chunk += 1

                self.text = _res.text


from types import FunctionType
from base_aios.validate.lang import lang
import re

HTTP_RE = re.compile(r'^((https|http|ftp|rtsp|mms)?:\/\/)[^\s]+')
TEL_RE = re.compile(r'^0?(13|14|15|17|18|19)[0-9]{9}$')
EMAIL_RE = re.compile(r'\w[-\w.+]*@([A-Za-z0-9][-A-Za-z0-9]+\.)+[A-Za-z]{2,14}')

class ParamsValid(object):

    def __init__(self, **args):
        self.v_map = {
            'type': self.validate_type,
            'required': self.validate_required,
            'minlen': self.validate_minlen,
            'maxlen': self.validate_maxlen,
            'max': self.validate_max,
            'min': self.validate_min,
            're': self.validate_re,
            'function': self.validate_function,
            'enum': self.validate_enum
        }
        self.fields = args or {}

    def validate(self, data={}):
        try:
            for field in self.fields:
                rules = self.fields.get(field)
                rules.setdefault('type', str)

                v_data = data.get(field)

                required = rules.get('required')
                if required is None and v_data is None:
                    continue
                for rule in rules:
                    v_rule = rules.get(rule)
                    fn = self.v_map.get(rule)
                    if fn:
                        is_valid, msg = fn(v_rule, field, v_data)
                        if not is_valid:
                            raise Exception('{} {}'.format(lang.resp('L_PARAMS_VERIFICATION_FAILED'), msg))
                    else:
                        raise Exception('{} [{}]'.format(lang.resp('L_UNKNOWN_VALIDATION_TYPE'), rule))
            return True, ''
        except Exception as err:
            return False, str(err)

    def validate_type(self, rule, key, value):
        if isinstance(value, rule):
            return True, ''
        else:
            try:
                rule(value)
                return True, ''
            except Exception as err:
                pass
            return False, '{}  {}'.format(lang.resp('L_DATA_TYPE_NOT_MATCH'), key)
                
    def validate_required(self, rule, key, value):
        if rule:
            if value is not None and value is not '':
                return True, ''
            else:
                return False, '{}  {}'.format(lang.resp('L_REQUIRED_ITEMS_CAN_NOT_BE_EMPTY'), key) 
        else:
            return True, ''

    def validate_minlen(self, rule, key, value):
        if rule <= len(value):
            return True, ''
        else:
            return False, '{}  {}'.format(lang.resp('L_LENGTH_OF_THE_STRING_EXCEEDS_THE_LIMIT'), key)

    def validate_maxlen(self, rule, key, value):
        if len(value) <= rule:
            return True, ''
        else:
            return False, '{}  {}'.format(lang.resp('L_LENGTH_OF_THE_STRING_EXCEEDS_THE_LIMIT'), key)

    def validate_min(self, rule, key, value):
        if rule <= float(value):
            return True, ''
        else:
            return False, '{}  {}'.format(lang.resp('L_VALUE_RANGE_EXCEEDS_THE_LIMIT'), key)

    def validate_max(self, rule, key, value):
        if float(value) <= rule:
            return True, ''
        else:
            return False, '{}  {}'.format(lang.resp('L_VALUE_RANGE_EXCEEDS_THE_LIMIT'), key)

    def validate_re(self, rule, key, value):
        if rule.match(value):
            return True, ''
        else:
            return False, '{}  {}'.format(lang.resp('L_DATA_FORMAT_ERROR'), key)

    def validate_function(self, rule, key, value):
        if isinstance(rule, FunctionType):
            if rule(value):
                return True, ''
        return False, '{}  {}'.format(lang.resp('L_CALLBACK_VALIDATE_ERROR'), key)

    def validate_enum(self, rule, key, value):
        if value in rule:
            return True, ''
        else:
            return False, '{}  {}'.format(lang.resp('L_ENUM_VALIDATE_ERROR'), key)
